@@ -43,7 +43,8 @@ typedef enum
   WNCK_WINDOW_STATE_STICKY                 = 1 << 6,
   WNCK_WINDOW_STATE_HIDDEN                 = 1 << 7,
   WNCK_WINDOW_STATE_FULLSCREEN             = 1 << 8,
-  WNCK_WINDOW_STATE_DEMANDS_ATTENTION      = 1 << 9
+  WNCK_WINDOW_STATE_DEMANDS_ATTENTION      = 1 << 9,
+  WNCK_WINDOW_STATE_URGENT                 = 1 << 10,
 } WnckWindowState;
 
 typedef enum
@@ -144,6 +145,7 @@ WnckClassGroup *wnck_window_get_class_group (WnckWindow *window);
 const char* wnck_window_get_session_id        (WnckWindow *window);
 const char* wnck_window_get_session_id_utf8   (WnckWindow *window);
 int         wnck_window_get_pid               (WnckWindow *window);
+gint        wnck_window_get_sort_order        (WnckWindow *window);
 
 WnckWindowType wnck_window_get_window_type    (WnckWindow *window);
 
@@ -154,9 +156,10 @@ gboolean wnck_window_is_maximized              (WnckWindow *window);
 gboolean wnck_window_is_shaded                 (WnckWindow *window);
 gboolean wnck_window_is_skip_pager             (WnckWindow *window);
 gboolean wnck_window_is_skip_tasklist          (WnckWindow *window);
-gboolean wnck_window_is_fullscreen                 (WnckWindow *window);
+gboolean wnck_window_is_fullscreen             (WnckWindow *window);
 gboolean wnck_window_is_sticky                 (WnckWindow *window);
-gboolean wnck_window_demands_attention         (WnckWindow *window);
+gboolean wnck_window_needs_attention           (WnckWindow *window);
+gboolean wnck_window_or_transient_needs_attention (WnckWindow *window);
 
 void wnck_window_set_skip_pager    (WnckWindow *window,
                                     gboolean skip);
@@ -166,9 +169,10 @@ void wnck_window_set_fullscreen (WnckWindow *window,
                                  gboolean fullscreen);
 
 void wnck_window_close                   (WnckWindow *window,
-					  guint32     timestamp);
+                                          guint32     timestamp);
 void wnck_window_minimize                (WnckWindow *window);
-void wnck_window_unminimize              (WnckWindow *window);
+void wnck_window_unminimize              (WnckWindow *window,
+                                          guint32     timestamp);
 void wnck_window_maximize                (WnckWindow *window);
 void wnck_window_unmaximize              (WnckWindow *window);
 void wnck_window_maximize_horizontally   (WnckWindow *window);
@@ -191,10 +195,12 @@ gboolean wnck_window_is_pinned (WnckWindow *window);
 void     wnck_window_pin       (WnckWindow *window);
 void     wnck_window_unpin     (WnckWindow *window);
 
-void     wnck_window_activate  (WnckWindow *window);
+void     wnck_window_activate  (WnckWindow *window,
+                                guint32     timestamp);
 gboolean wnck_window_is_active (WnckWindow *window);
 gboolean wnck_window_is_most_recently_activated (WnckWindow *window);
-void     wnck_window_activate_transient (WnckWindow *window);
+void     wnck_window_activate_transient (WnckWindow *window,
+                                         guint32     timestamp);
 gboolean wnck_window_transient_is_active (WnckWindow *window);
 
 GdkPixbuf* wnck_window_get_icon      (WnckWindow *window);
